﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class FlightEngineer : EngineerModule
    {
        Vessel activeVessel;
        bool showDisplay = true;

        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            if (state != StartState.Editor && vessel.isActiveVessel)
            {
                activeVessel = FlightGlobals.ActiveVessel;
                IsPrimary = CheckIsPrimary("FlightEngineer", activeVessel.parts);

                if (IsPrimary)
                {
                    winPos = new Rect(UnityEngine.Screen.width - 275, 0, 0, 0);
                    RenderingManager.AddToPostDrawQueue(0, DrawGUI);
                }

                Debug("FlightEngineer: OnStart");
            }

            Debug("FlightEngineer: OnStart(" + state + ")");
        }

        public override void OnUpdate()
        {
            base.OnUpdate();
            OnStart(state);
            Debug("FlightEngineer: OnUpdate");
        }

        public override void DrawGUI()
        {
            base.DrawGUI();
            winStyle.fixedWidth = 275;
            dataStyle.alignment = TextAnchor.UpperRight;

            if (this.part == null || !this.vessel.isActiveVessel)
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
                return;
            }

            ship.Update(activeVessel.parts);
            winPos = GUILayout.Window(3, winPos, MainWindow, "Flight Engineer", winStyle);

            //Debug("FlightEngineer: DrawGUI");
        }

        public void MainWindow(int windowId)
        {
            string btnText = "Show";
            if (showDisplay)
                btnText = "Hide";

            GUILayout.BeginHorizontal();
            if (GUILayout.Button(btnText, buttonStyle))
            {
                showDisplay = !showDisplay;
                winPos.height = 0;
            }
            GUILayout.EndHorizontal();

            if (showDisplay)
                DrawDisplay();

            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawDisplay()
        {
            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Altitude (True)", headStyle);
            GUILayout.Label("Apoapsis Height", headStyle);
            GUILayout.Label("Time to Apoapsis", headStyle);
            GUILayout.Label("Periapsis Height", headStyle);
            GUILayout.Label("Time to Periapsis", headStyle);
            GUILayout.Label("Inclination", headStyle);
            GUILayout.Label("Eccentricity", headStyle);
            GUILayout.Label("Orbital Period", headStyle);
            GUILayout.Label("Semi-major Axis", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatDistance(activeVessel.mainBody.GetAltitude(activeVessel.CoM) - activeVessel.terrainAltitude), dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.orbit.ApA), dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToAp.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.orbit.PeA), dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToPe.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(activeVessel.orbit.inclination.ToString("0.000") + "°", dataStyle);
            GUILayout.Label(activeVessel.orbit.eccentricity.ToString("0.000000"), dataStyle);
            GUILayout.Label(activeVessel.orbit.period.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.orbit.semiMajorAxis), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Vertical Speed", headStyle);
            GUILayout.Label("Horizontal Speed", headStyle);
            GUILayout.Label("G-Force", headStyle);
            GUILayout.Label("Longitude", headStyle);
            GUILayout.Label("Latitude", headStyle);
            GUILayout.Label("Atmospheric Density", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatSpeed(activeVessel.verticalSpeed), dataStyle);
            GUILayout.Label(FormatSpeed(activeVessel.horizontalSrfSpeed), dataStyle);
            GUILayout.Label(activeVessel.geeForce.ToString("0.000"), dataStyle);
            GUILayout.Label(activeVessel.longitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.latitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.atmDensity.ToString("0.000000"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V (Stage)", headStyle);
            GUILayout.Label("Delta-V (Total)", headStyle);
            GUILayout.Label("Fuel", headStyle);
            GUILayout.Label("Mass", headStyle);
            GUILayout.Label("TWR", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(GetStage(activeVessel.currentStage).DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).TotalDeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Fuel.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalFuel.ToString("0.00"), dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Mass.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalMass.ToString("0.00"), dataStyle);
            GUILayout.Label((GetStage(activeVessel.currentStage).Thrust / (GetStage(activeVessel.currentStage).TotalMass * (activeVessel.mainBody.gravParameter / Math.Pow(activeVessel.mainBody.Radius, 2)))).ToString("0.00"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private string FormatSpeed(double number)
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }

            string notation = "m/s";
            string strNumber = number.ToString("#,##0.0");

            if (number < 1)
            {
                number *= 100;
                if (number >= 1)
                {
                    notation = "cm/s";
                    strNumber = number.ToString("0.00");
                }

                if (number < 1)
                {
                    notation = "mm/s";
                    number *= 100;
                    strNumber = number.ToString("0");
                }
            }

            if (number >= 1000)
            {
                notation = "km/s";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            return strNumber + notation;
        }

        private string FormatDistance(double number)
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }


            string notation = "m";
            string strNumber = number.ToString("#,##0");

            if (number < 100)
                strNumber = number.ToString("0.000");

            if (number >= 10000)
            {
                notation = "km";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (number >= 10000)
            {
                notation = "Mm";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            return strNumber + notation;
        }

        private void CheckGUI()
        {
        }
    }
}
